/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer {
import flash.errors.IllegalOperationError;
import flash.utils.IDataInput;
import flash.utils.IDataOutput;
import flash.utils.IExternalizable;

/**
 * @author Loic Albertin
 */
[Bindable]
[RemoteClass(alias="javax.xml.namespace.QName")]
public class FlexQName implements IExternalizable {

  private var _namespaceURI:String;
  private var _localPart:String;

  public function FlexQName() {
  }

  public static function valueOf(qNameAsString:String):FlexQName {
    if (qNameAsString == null || qNameAsString == "") {
      throw new IllegalOperationError("Invalid argument: QName as string is null or empty");
    }
    var qName:FlexQName = new FlexQName();
    if (qNameAsString.charAt(0) != "{") {
      qName.localPart = qNameAsString;
      qName.namespaceURI = "";
    } else {
      var namespaceEndIndex:int = qNameAsString.lastIndexOf("}");
      if (namespaceEndIndex == -1) {
        throw new IllegalOperationError("Invalid argument: missing namespace uri separator");
      }
      qName.namespaceURI = qNameAsString.substring(1, namespaceEndIndex);
      qName.localPart = qNameAsString.substring(namespaceEndIndex + 1);
    }
    return qName;
  }

  public function get namespaceURI():String {
    return _namespaceURI;
  }

  public function set namespaceURI(value:String):void {
    _namespaceURI = value;
  }

  public function get localPart():String {
    return _localPart;
  }

  public function set localPart(value:String):void {
    _localPart = value;
  }

  public function readExternal(input:IDataInput):void {
    _namespaceURI = input.readObject() as String;
    _localPart = input.readObject() as String;
  }

  public function writeExternal(output:IDataOutput):void {
    output.writeObject(_namespaceURI);
    output.writeObject(_localPart);
  }

  public function toString():String {
    var ns:String = "";
    if (this.namespaceURI != null && this.namespaceURI != "") {
      ns += "{" + this.namespaceURI + "}";
    }
    return ns + this.localPart;
  }

  public function equals(other:Object):Boolean {
    if (other == null || !(other is FlexQName)) {
      return false;
    }
    var otherQName:FlexQName = other as FlexQName;
    return otherQName.namespaceURI == this.namespaceURI && otherQName.localPart == this.localPart;
  }
}
}
