package org.ow2.orchestra.designer.bpmn.model.utils {
import mx.collections.ArrayCollection;
import mx.collections.ListCollectionView;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.data.AbstractDataModel;
import org.ow2.orchestra.designer.bpmn.model.data.AbstractExpressionElementModel;
import org.ow2.orchestra.designer.bpmn.model.data.DataInputModel;

import org.ow2.orchestra.designer.bpmn.model.data.DataMappingModel;
import org.ow2.orchestra.designer.bpmn.model.data.DataObjectModel;
import org.ow2.orchestra.designer.bpmn.model.data.ExpressionDataInputModel;
import org.ow2.orchestra.designer.bpmn.model.data.ExpressionDataObjectModel;
import org.ow2.orchestra.designer.bpmn.model.data.type.DataTypeModel;
import org.ow2.orchestra.designer.bpmn.model.data.type.XSDSequenceType;
import org.ow2.orchestra.designer.bpmn.model.data.type.XSDElementModel;

/**
 * @author Loic Albertin
 */
public class DataUtil {
  public function DataUtil() {
  }

  public static function hasDataInputsNotInDataMappingModel(dataInputs:ListCollectionView, dataMappingModel:DataMappingModel):Boolean {
    for each (var dataInputModel:DataInputModel in dataInputs) {
      if (!DataUtil.isDataPresent(dataMappingModel, dataInputModel) || DataUtil.hasSubElementsNotInMapping(dataInputModel, dataMappingModel)) {
        return true;
      }
    }
    return false;
  }

  private static function hasSubElementsNotInMapping(dataInputModel:DataInputModel, dataMappingModel:DataMappingModel):Boolean {
    if (dataInputModel.type is XSDSequenceType ||
        dataInputModel.type is XSDElementModel && XSDElementModel(dataInputModel.type).type is XSDSequenceType) {
      var sequenceElements:ListCollectionView;
      if (dataInputModel.type is XSDSequenceType) {
        sequenceElements = XSDSequenceType(dataInputModel.type).elements
      } else {
        sequenceElements = XSDSequenceType(XSDElementModel(dataInputModel.type).type).elements;
      }
      for each (var dataTypeModel:DataTypeModel in sequenceElements) {
        if (!DataUtil.isDataPresent(dataMappingModel, dataInputModel, dataTypeModel)) {
          return true;
        }
      }
    }
    return false;
  }

  public static function isDataPresent(dataMappingModel:DataMappingModel, dataModel:AbstractDataModel, type:DataTypeModel = null):Boolean {
    for each (var abstractElement:AbstractElement in dataMappingModel.elements) {
      if (abstractElement is AbstractExpressionElementModel) {
        var exprModel:AbstractExpressionElementModel = abstractElement as AbstractExpressionElementModel;
        var currentDataModel:AbstractDataModel = null;
        if (exprModel is ExpressionDataInputModel) {
          if (dataModel is DataObjectModel) {
            //Not same type
            continue;
          }
          currentDataModel = ExpressionDataInputModel(exprModel).dataInputModel;
        } else if (exprModel is ExpressionDataObjectModel) {
          if (dataModel is DataInputModel) {
            //Not same type
            continue;
          }
          currentDataModel = ExpressionDataObjectModel(exprModel).dataObjectModel;
        }
        if (currentDataModel != null && currentDataModel.name == dataModel.name) {
          if ((type == null &&  exprModel.childNodeSelector == null) ||
              (type != null && exprModel.childNodeSelector == type.QName.toString())) {
            return true;
          }
        }
      }
    }
    return false;
  }
}
}
